Module: ActionView::Helpers::RenderingHelper
- Included in:
- ActionView::Helpers
- Defined in:
- actionview/lib/action_view/helpers/rendering_helper.rb
Overview
Action View Rendering
Implements methods that allow rendering from a view context. In order to use this module, all you need is to implement view_renderer that returns an ActionView::Renderer object.
Instance Method Summary collapse
-
#_layout_for(*args, &block) ⇒ Object
Overwrites _layout_for in the context object so it supports the case a block is passed to a partial.
-
#render(options = {}, locals = {}, &block) ⇒ Object
Returns the result of a render that's dictated by the options hash.
Instance Method Details
#_layout_for(*args, &block) ⇒ Object
Overwrites _layout_for in the context object so it supports the case a block is passed to a partial. Returns the contents that are yielded to a layout, given a name or a block.
You can think of a layout as a method that is called with a block. If the user calls
yield :some_name, the block, by default, returns content_for(:some_name).
If the user calls simply yield, the default block returns content_for(:layout).
The user can override this default by passing a block to the layout:
# The template
<%= render layout: "my_layout" do %>
Content
<% end %>
# The layout
<html>
<%= yield %>
</html>
In this case, instead of the default block, which would return content_for(:layout), this method returns the block that was passed in to render :layout, and the response would be
ContentFinally, the block can take block arguments, which can be passed in by yield:
# The template
<%= render layout: "my_layout" do |customer| %>
Hello <%= customer.name %>
<% end %>
# The layout
<html>
<%= yield Struct.new(:name).new("David") %>
</html>
In this case, the layout would receive the block passed into render :layout, and the struct specified would be passed into the block as an argument. The result would be
Hello David
89 90 91 92 93 94 95 96 97 |
# File 'actionview/lib/action_view/helpers/rendering_helper.rb', line 89 def _layout_for(*args, &block) name = args.first if block && !name.is_a?(Symbol) capture(*args, &block) else super end end |
#render(options = {}, locals = {}, &block) ⇒ Object
Returns the result of a render that's dictated by the options hash. The primary options are:
- :partial - See ActionView::PartialRenderer.
- :file - Renders an explicit template file (this used to be the old default), add :locals to pass in those.
- :inline - Renders an inline template similar to how it's done in the controller.
- :text - Renders the text passed in out.
- :plain - Renders the text passed in out. Setting the content type as text/plain.
- :html - Renders the HTML safe string passed in out, otherwise performs HTML escape on the string first. Setting the content type as text/html.
- :body - Renders the text passed in, and inherits the content type of text/plain from ActionDispatch::Response object.
If no options hash is passed or :update specified, the default is to render a partial and use the second parameter as the locals hash.
28 29 30 31 32 33 34 35 36 37 38 39 |
# File 'actionview/lib/action_view/helpers/rendering_helper.rb', line 28 def render( = {}, locals = {}, &block) case when Hash if block_given? view_renderer.render_partial(self, .merge(partial: [:layout]), &block) else view_renderer.render(self, ) end else view_renderer.render_partial(self, partial: , locals: locals, &block) end end |