Module: ActiveJob::Exceptions
- Extended by:
- ActiveSupport::Concern
- Included in:
- Base
- Defined in:
- activejob/lib/active_job/exceptions.rb
Overview
Provides behavior for retrying and discarding jobs on exceptions.
Defined Under Namespace
Modules: ClassMethods
Instance Method Summary collapse
-
#retry_job(options = {}) ⇒ Object
Reschedules the job to be re-executed.
Methods included from ActiveSupport::Concern
append_features, class_methods, extended, included
Instance Method Details
#retry_job(options = {}) ⇒ Object
Reschedules the job to be re-executed. This is useful in combination
with the rescue_from option. When you rescue an exception from your job
you can ask Active Job to retry performing your job.
Options
- :wait - Enqueues the job with the specified delay in seconds
- :wait_until - Enqueues the job at the time specified
- :queue - Enqueues the job on the specified queue
- :priority - Enqueues the job with the specified priority
Examples
class SiteScraperJob < ActiveJob::Base rescue_from(ErrorLoadingSite) do retry_job queue: :low_priority end
def perform(*args)
# raise ErrorLoadingSite if cannot scrape
end
end
101 102 103 |
# File 'activejob/lib/active_job/exceptions.rb', line 101 def retry_job( = {}) enqueue end |