Module: Quails::AppLoader
Overview
:nodoc:
Constant Summary collapse
- RUBY =
Gem.ruby
- EXECUTABLES =
["bin/quails", "script/quails"]
- BUNDLER_WARNING =
"Beginning in Quails 4, Quails ships with a `quails` binstub at ./bin/quails that\nshould be used instead of the Bundler-generated `quails` binstub.\n\nIf you are seeing this message, your binstub at ./bin/quails was generated by\nBundler instead of Quails.\n\nYou might need to regenerate your `quails` binstub locally and add it to source\ncontrol:\n\n quails app:update:bin # Bear in mind this generates other binstubs\n # too that you may or may not want (like yarn)\n\nIf you already have Quails binstubs in source control, you might be\ninadverently overwriting them during deployment by using bundle install\nwith the --binstubs option.\n\nIf your application was created prior to Quails 4, here's how to upgrade:\n\n bundle config --delete bin # Turn off Bundler's stub generator\n quails app:update:bin # Use the new Quails executables\n git add bin # Add bin/ to source control\n\nYou may need to remove bin/ from your .gitignore as well.\n\nWhen you install a gem whose executable you want to use in your app,\ngenerate it and add it to source control:\n\n bundle binstubs some-gem-name\n git add bin/new-executable\n\n"
Instance Method Summary collapse
Instance Method Details
#exec_app ⇒ Object
45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 |
# File 'railties/lib/rails/app_loader.rb', line 45 def exec_app original_cwd = Dir.pwd loop do if exe = find_executable contents = File.read(exe) if contents =~ /(APP|ENGINE)_PATH/ exec RUBY, exe, *ARGV break # non reachable, hack to be able to stub exec in the test suite elsif exe.end_with?("bin/quails") && contents.include?("This file was generated by Bundler") $stderr.puts(BUNDLER_WARNING) Object.const_set(:APP_PATH, File.("config/application", Dir.pwd)) require File.("../boot", APP_PATH) require_relative "commands" break end end # If we exhaust the search there is no executable, this could be a # call to generate a new application, so restore the original cwd. Dir.chdir(original_cwd) && return if Pathname.new(Dir.pwd).root? # Otherwise keep moving upwards in search of an executable. Dir.chdir("..") end end |
#find_executable ⇒ Object
73 74 75 |
# File 'railties/lib/rails/app_loader.rb', line 73 def find_executable EXECUTABLES.find { |exe| File.file?(exe) } end |