Method: Enumerable#sum
- Defined in:
-
activesupport/lib/active_support/core_ext/enumerable.rb,
activesupport/lib/active_support/core_ext/enumerable.rb
Calculates a sum from the elements.
payments.sum { |p| p.price * p.tax_rate }
payments.sum(&:price)
The latter is a shortcut for:
payments.inject(0) { |sum, p| sum + p.price }
It can also calculate the sum without the use of a block.
[5, 15, 10].sum # => 30
['foo', 'bar'].sum # => "foobar"
[[1, 2], [3, 1, 5]].sum # => [1, 2, 3, 1, 5]
The default sum of an empty list is zero. You can override this default:
[].sum(Payment.new(0)) { |i| i.amount } # => Payment.new(0)
33 34 35 36 37 38 39 40 41 |
# File 'activesupport/lib/active_support/core_ext/enumerable.rb', line 33 def sum(identity = nil, &block) if identity _original_sum_with_required_identity(identity, &block) elsif block_given? map(&block).sum(identity) else inject(:+) || 0 end end |